<?php
/*
 * @package     RadicalMart Package
 * @subpackage  plg_system_radicalmart
 * @version     __DEPLOY_VERSION__
 * @author      RadicalMart Team - radicalmart.ru
 * @copyright   Copyright (c) 2024 RadicalMart. All rights reserved.
 * @license     GNU/GPL license: https://www.gnu.org/copyleft/gpl.html
 * @link        https://radicalmart.ru/
 */

namespace MSGJoomla\Plugin\RadicalMart\FilterIndex\Extension;

\defined('_JEXEC') or die;

use Joomla\Application\ApplicationEvents;
use Joomla\CMS\Extension\ModuleInterface;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Factory\MVCFactoryAwareTrait;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\Component\RadicalMart\Administrator\Helper\CommandsHelper;
use Joomla\Database\DatabaseAwareTrait;
use Joomla\Event\SubscriberInterface;
use Joomla\Filesystem\File;
use Joomla\Filesystem\Folder;
use Joomla\Filesystem\Path;
use Joomla\Registry\Registry;
use Joomla\Event\EventInterface;
use Joomla\Utilities\ArrayHelper;
use MSGJoomla\Plugin\RadicalMart\FilterIndex\Console\MSGFilterIndex;
use MSGJoomla\Module\RMFilter\Site\Helper\FilterHelper;

class FilterIndex extends CMSPlugin implements SubscriberInterface
{
	use MVCFactoryAwareTrait;
	use DatabaseAwareTrait;

	/**
	 * Load the language file on instantiation.
	 *
	 * @var    bool
	 *
	 * @since  1.0.0
	 */
	protected $autoloadLanguage = true;

	/**
	 * Returns an array of events this subscriber will listen to.
	 *
	 * @return  array
	 *
	 * @since   1.0.0
	 */
	public static function getSubscribedEvents(): array
	{
		return [
            ApplicationEvents::BEFORE_EXECUTE       => 'registerCLICommands',
            'onRadicalMartRegisterCLICommands' => 'registerCLICommands',
            'onRadicalMartGetFilterParams' => 'onRadicalMartGetFilterParams',
            'onRadicalMartGetAdministratorCommands' => 'onRadicalMartGetAdministratorCommands',
		];
	}

    /**
     * Constructor.
     *
     * @param   DispatcherInterface  &$subject  The object to observe.
     * @param   array                 $config   An optional associative array of configuration settings.
     *
     * @since  1.0.0
     */
    public function __construct(&$subject, $config = [])
    {
        $this->input = Factory::getApplication()->getInput();
        parent::__construct($subject, $config);
    }

    /**
     * Method to register CLI commands.
     *
     * @param   array     $commands  Updated commands array.
     * @param   Registry  $params    RadicalMart params.
     *
     * @since 1.0.0
     */
    public function registerCLICommands(array &$commands, Registry $params)
    {
        $files    = Folder::files(Path::clean(JPATH_PLUGINS . '/' . $this->_type . '/' . $this->_name .'/src/Console'), '.php');

        foreach ($files as $file)
        {
            $commands[] = 'MSGJoomla\\Plugin\\RadicalMart\\FilterIndex\\Console\\' . File::stripExt($file);
        }
    }

    public function onRadicalMartGetFilterParams(Registry &$params)
    {
        $params->merge($this->params);
    }

    /**
     * Listener for the `onRadicalMartGetAdministratorCommands` event.
     *
     * @param   string|null  $context   Context selector string.
     * @param   array|null   $commands  Commands data array.
     *
     * @since  2.0.0
     */
    public function onRadicalMartGetAdministratorCommands(?string $context = null, ?array &$commands = [])
    {
        $this->loadStandardRadicalMartAdministratorCommands($context, $commands);
    }

    /**
     * Method to add administrator commands config.
     *
     * @param   string|null  $context   Context selector string.
     * @param   array        $commands  Administrator commands array.
     *
     * @since  2.0.0
     */
    protected function loadStandardRadicalMartAdministratorCommands(?string $context = null, array &$commands = [])
    {
        if (in_array($context, ['com_radicalmart.commands', 'com_radicalmart.fields'])) {
            $commands['radicalmart_filter:index'] = [
                'command' => 'radicalmart_filter:index',
                'text' => 'PLG_RADICALMART_MSGIF_COMMANDS_FILTER_INDEXING',
                'method' => function (string $task) {
                    if ($task === 'total')
                    {
                        $cacheModel = Factory::getApplication()->bootComponent('com_cache')->getMVCFactory()
                            ->createModel('Cache', 'Administrator', ['ignore_request' => true]);

                        if (!$cacheModel->clean('mod_msg_rmf')) {
                            throw new \Exception(Text::_('PLG_RADICALMART_MSGIF_COMMANDS_NOT_CLEAR_CACHE'));
                        }

                        return $this->getTotal('#__radicalmart_categories');
                    }

                    $pk = $this->input->getInt('pk', 0);
                    if (empty($pk))
                    {
                        throw new \Exception(Text::_('PLG_RADICALMART_MSGIF_COMMANDS_PK_NOT_FOUND'), 404);
                    }

                    $params = new Registry();
                    $params->set('fields', MSGFilterIndex::getFields());

                    $params->set('category_id', $pk);

                    FilterHelper::getForm($params);

                    return $this->getNextPrimaryKey('#__radicalmart_categories');
                },
            ];
        }
    }

    /**
     * Method to get total items, and first item primary key.
     *
     * @param   string  $table   Table name.
     * @param   string  $column  Primary key column.
     *
     * @return array Result data.
     *
     * @since 2.0.0
     */
    protected function getTotal(string $table = '', string $column = 'id'): array
    {
        $total = 0;
        $next  = 0;

        $all     = ($this->input->getInt('all', 0) === 1);
        $item_pk = $this->input->getInt('item_pk', 0);
        if (!$all && $item_pk === 0)
        {
            if (!empty($this->input->get('cid', [], 'array')))
            {
                $pks   = ArrayHelper::toInteger($this->input->get('cid', [], 'array'));
                $total = count($pks);
                $next  = (int) array_shift($pks);
            }
            elseif ($this->input->get('jform', [], 'array'))
            {
                $data = $this->input->get('jform', [], 'array');
                if (!empty($data['id']))
                {
                    $total = 1;
                    $next  = (int) $data['id'];
                }
            }
        }
        elseif (!$all && $item_pk > 0)
        {
            $total = 1;
            $next  = $item_pk;
        }

        if (empty($total))
        {
            $total = CommandsHelper::getTotalItems($table, $column);
        }

        if (empty($next))
        {
            $next = CommandsHelper::getNextPrimaryKey($table, 0, $column);
        }

        return [
            'total' => $total,
            'next'  => $next,
        ];
    }

    /**
     * Method to get current and next item primary keys.
     *
     * @param   string  $table   Table name.
     * @param   string  $column  Primary key column.
     *
     * @return array Result data.
     *
     * @since 2.0.0
     */
    protected function getNextPrimaryKey(string $table = '', string $column = 'id'): array
    {
        $current = $this->input->getInt('pk', 0);
        $result  = [
            'current' => $current,
            'next'    => 0,
        ];

        $all     = ($this->input->getInt('all', 0) === 1);
        $item_pk = $this->input->getInt('item_pk', 0);
        if (!$all && $item_pk === 0)
        {
            if ($this->input->get('jform', [], 'array'))
            {
                $data = $this->input->get('jform', [], 'array');
                if (!empty($data['id']) && $current === (int) $data['id'])
                {
                    return $result;
                }
            }

            if (!empty($this->input->get('cid', [], 'array')))
            {
                $pks     = ArrayHelper::toInteger($this->input->get('cid', [], 'array'));
                $next    = 0;
                $getNext = false;
                foreach ($pks as $pk)
                {
                    if ($getNext)
                    {
                        $next = (int) $pk;
                        break;
                    }

                    if ((int) $pk === $current)
                    {
                        $getNext = true;
                    }
                }

                $result['next'] = $next;

                return $result;

            }
        }
        elseif (!$all && $item_pk > 0)
        {
            if ($current === $item_pk)
            {
                $result['next'] = 0;

                return $result;
            }
            else
            {
                $result['next'] = $item_pk;

                return $result;
            }
        }

        $result['next'] = CommandsHelper::getNextPrimaryKey($table, $current, $column);

        return $result;
    }
}
